import datetime as dt
from dataclasses import field
from enum import Enum
from typing import List, Optional

from pydantic.dataclasses import dataclass

__NAMESPACE__ = "http://am.oxm.api.portal.qualys.com/v2"


class TagRuleType(Enum):
    STATIC = "STATIC"
    GROOVY = "GROOVY"
    OS_REGEX = "OS_REGEX"
    NETWORK_RANGE = "NETWORK_RANGE"
    NETWORK_RANGE_ENHANCED = "NETWORK_RANGE_ENHANCED"
    NAME_CONTAINS = "NAME_CONTAINS"
    INSTALLED_SOFTWARE = "INSTALLED_SOFTWARE"
    OPEN_PORTS = "OPEN_PORTS"
    VULN_EXIST = "VULN_EXIST"
    ASSET_SEARCH = "ASSET_SEARCH"
    CLOUD_ASSET = "CLOUD_ASSET"
    BUSINESS_INFORMATION = "BUSINESS_INFORMATION"
    GLOBAL_ASSET_VIEW = "GLOBAL_ASSET_VIEW"
    TAG_SET = "TAG_SET"


@dataclass
class TagSimple:
    id: Optional[int] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v2",
            "required": True,
        },
    )
    name: Optional[str] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v2",
            "required": True,
        },
    )


@dataclass
class TagSimpleList:
    tag_simple: List[TagSimple] = field(
        default_factory=list,
        metadata={
            "name": "TagSimple",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v2",
        },
    )


@dataclass
class TagSimpleObj:
    tag_simple: Optional[TagSimple] = field(
        default=None,
        metadata={
            "name": "TagSimple",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v2",
        },
    )


@dataclass
class TagSimpleQlist:
    class Meta:
        name = "TagSimpleQList"

    count: Optional[int] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v2",
        },
    )
    list_value: List[TagSimpleObj] = field(
        default_factory=list,
        metadata={
            "name": "list",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v2",
        },
    )
    set: Optional[TagSimpleList] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v2",
        },
    )
    add: Optional[TagSimpleList] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v2",
        },
    )
    remove: Optional[TagSimpleList] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v2",
        },
    )
    update: Optional[TagSimpleList] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v2",
        },
    )


@dataclass
class Tag(TagSimple):
    parent_tag_id: Optional[int] = field(
        default=None,
        metadata={
            "name": "parentTagId",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v2",
        },
    )
    created: Optional[dt.datetime] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v2",
            "format": "%Y-%m-%dT%H:%M:%SZ",
        },
    )
    modified: Optional[dt.datetime] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v2",
            "format": "%Y-%m-%dT%H:%M:%SZ",
        },
    )
    color: Optional[str] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v2",
            "pattern": r"#[0-9a-fA-F]{3}([0-9a-fA-F]{3})?",
        },
    )
    rule_text: Optional[str] = field(
        default=None,
        metadata={
            "name": "ruleText",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v2",
        },
    )
    rule_type: Optional[TagRuleType] = field(
        default=None,
        metadata={
            "name": "ruleType",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v2",
        },
    )
    src_asset_group_id: Optional[int] = field(
        default=None,
        metadata={
            "name": "srcAssetGroupId",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v2",
        },
    )
    src_business_unit_id: Optional[int] = field(
        default=None,
        metadata={
            "name": "srcBusinessUnitId",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v2",
        },
    )
    src_operating_system_name: Optional[str] = field(
        default=None,
        metadata={
            "name": "srcOperatingSystemName",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v2",
        },
    )
    provider: Optional[str] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v2",
        },
    )
    children: Optional[TagSimpleQlist] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v2",
        },
    )
    criticality_score: Optional[int] = field(
        default=None,
        metadata={
            "name": "criticalityScore",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v2",
        },
    )
    description: Optional[str] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v2",
        },
    )



# mypy: ignore-errors
# type: ignore

import inspect
import ipaddress
import os
import sys
import unittest

import sqlalchemy.orm as orm

currentdir = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))
parentdir = os.path.dirname(currentdir)
sys.path.insert(0, parentdir)


import qualyspy.models.vmdr.host_list_orm as host_list_orm  # noqa: E402
import qualyspy.models.vmdr.host_list_vm_detection_orm as host_list_vm_detection_orm  # noqa: E402
from qualyspy import vmdr  # noqa: E402


class TestVMDR(unittest.TestCase):
    def test_vmdr_host_list_det_orm_load(self):
        vmdr_orm = vmdr.HostListDetectionORM(echo=True)
        vmdr_orm.load()
        with orm.Session(vmdr_orm.engine) as session:
            host_list = (
                session.query(host_list_vm_detection_orm.Host)
                .where(host_list_vm_detection_orm.Host.id == 11619472)
                .all()
            )
            self.assertEqual(host_list[0].ip, ipaddress.ip_address("172.16.76.84"))

    def test_vmdr_host_list_orm_load(self):
        vmdr_orm = vmdr.HostListORM(echo=True)
        vmdr_orm.load()
        with orm.Session(vmdr_orm.engine) as session:
            host_list = (
                session.query(host_list_orm.Host)
                .where(host_list_orm.Host.id == 11619472)
                .all()
            )
            self.assertEqual(host_list[0].ip, ipaddress.ip_address("172.16.76.84"))


class TestKnowledgebase(unittest.TestCase):
    def test_vmdr_knowledgebase(self):
        api = vmdr.VmdrAPI()
        knowledgebase = api.knowledgebase(ids=105943)
        print("test")
        self.assertEqual(knowledgebase.vuln[0].qid, 105943)


if __name__ == "__main__":
    unittest.main()



from dataclasses import field
from typing import Optional

from pydantic.dataclasses import dataclass

__NAMESPACE__ = "http://am.oxm.api.portal.qualys.com/v3"


@dataclass
class AzureAuthRecord:
    application_id: Optional[str] = field(
        default=None,
        metadata={
            "name": "applicationId",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    directory_id: Optional[str] = field(
        default=None,
        metadata={
            "name": "directoryId",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    authentication_key: Optional[str] = field(
        default=None,
        metadata={
            "name": "authenticationKey",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    subscription_id: Optional[str] = field(
        default=None,
        metadata={
            "name": "subscriptionId",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )


@dataclass
class AzureAuthRecordSimple(AzureAuthRecord):
    id: Optional[int] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )



import datetime as dt
from dataclasses import field
from enum import Enum
from typing import List, Optional

from pydantic.dataclasses import dataclass

__NAMESPACE__ = "http://am.oxm.api.portal.qualys.com/v3"


class AssetSourceStateCode(Enum):
    PENDING = "PENDING"
    RUNNING = "RUNNING"
    SHUTTING_DOWN = "SHUTTING_DOWN"
    TERMINATED = "TERMINATED"
    STOPPING = "STOPPING"
    STOPPED = "STOPPED"
    DEALLOCATED = "DEALLOCATED"
    DEALLOCATING = "DEALLOCATING"
    CREATING = "CREATING"
    STARTING = "STARTING"
    DELETING = "DELETING"
    DELETED = "DELETED"
    UPDATING = "UPDATING"
    FAILED = "FAILED"
    MOVING = "MOVING"
    PROVISIONING = "PROVISIONING"
    CREATING_IMAGE = "CREATING_IMAGE"
    TERMINATING = "TERMINATING"
    SUCCEEDED = "SUCCEEDED"
    UNSUPPORTED = "UNSUPPORTED"


class AssetSourceType(Enum):
    EC2 = "EC2"
    LDAP = "LDAP"
    AZURE = "AZURE"
    GCP = "GCP"
    IBM = "IBM"
    OCI = "OCI"
    VMWARE_VSPHERE = "VMWARE_VSPHERE"


@dataclass
class AzureTags:
    key: Optional[str] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
            "required": True,
        },
    )
    value: Optional[str] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
            "required": True,
        },
    )


@dataclass
class Ec2Tags:
    class Meta:
        name = "EC2Tags"

    key: Optional[str] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
            "required": True,
        },
    )
    value: Optional[str] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
            "required": True,
        },
    )


@dataclass
class GcpTags:
    key: Optional[str] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
            "required": True,
        },
    )
    value: Optional[str] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
            "required": True,
        },
    )


@dataclass
class IbmTags:
    key: Optional[str] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
            "required": True,
        },
    )
    value: Optional[str] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
            "required": True,
        },
    )


@dataclass
class OciTag:
    key: Optional[str] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
            "required": True,
        },
    )
    value: Optional[str] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
            "required": True,
        },
    )
    type_value: Optional[str] = field(
        default=None,
        metadata={
            "name": "type",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
            "required": True,
        },
    )
    namespace: Optional[str] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
            "required": True,
        },
    )


@dataclass
class OciVnic:
    oci_id: Optional[str] = field(
        default=None,
        metadata={
            "name": "ociId",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
            "required": True,
        },
    )
    vnic_id: Optional[str] = field(
        default=None,
        metadata={
            "name": "vnicId",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
            "required": True,
        },
    )
    vlan_tag: Optional[int] = field(
        default=None,
        metadata={
            "name": "vlanTag",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
            "required": True,
        },
    )
    virtual_router_ip: Optional[str] = field(
        default=None,
        metadata={
            "name": "virtualRouterIp",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
            "required": True,
        },
    )
    subnet_cidr_block: Optional[str] = field(
        default=None,
        metadata={
            "name": "subnetCidrBlock",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
            "required": True,
        },
    )
    private_ip_address: Optional[str] = field(
        default=None,
        metadata={
            "name": "privateIpAddress",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
            "required": True,
        },
    )
    mac_address: Optional[str] = field(
        default=None,
        metadata={
            "name": "macAddress",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
            "required": True,
        },
    )
    vcn_id: Optional[str] = field(
        default=None,
        metadata={
            "name": "vcnId",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
            "required": True,
        },
    )
    vcn_name: Optional[str] = field(
        default=None,
        metadata={
            "name": "vcnName",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
            "required": True,
        },
    )
    vcn_compartment_id: Optional[str] = field(
        default=None,
        metadata={
            "name": "vcnCompartmentId",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
            "required": True,
        },
    )
    subnet_id: Optional[str] = field(
        default=None,
        metadata={
            "name": "subnetId",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
            "required": True,
        },
    )
    subnet_name: Optional[str] = field(
        default=None,
        metadata={
            "name": "subnetName",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
            "required": True,
        },
    )
    public_ip: Optional[str] = field(
        default=None,
        metadata={
            "name": "publicIp",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
            "required": True,
        },
    )


@dataclass
class AssetSource:
    asset_id: Optional[int] = field(
        default=None,
        metadata={
            "name": "assetId",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    type_value: Optional[AssetSourceType] = field(
        default=None,
        metadata={
            "name": "type",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    first_discovered: Optional[dt.datetime] = field(
        default=None,
        metadata={
            "name": "firstDiscovered",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
            "format": "%Y-%m-%dT%H:%M:%SZ",
        },
    )
    last_updated: Optional[dt.datetime] = field(
        default=None,
        metadata={
            "name": "lastUpdated",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
            "format": "%Y-%m-%dT%H:%M:%SZ",
        },
    )


@dataclass
class AzureTagsQlist:
    class Meta:
        name = "AzureTagsQList"

    count: Optional[int] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    list_value: List[AzureTags] = field(
        default_factory=list,
        metadata={
            "name": "list",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    set: List[AzureTags] = field(
        default_factory=list,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    add: List[AzureTags] = field(
        default_factory=list,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    remove: List[AzureTags] = field(
        default_factory=list,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    update: List[AzureTags] = field(
        default_factory=list,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )


@dataclass
class Ec2TagsQlist:
    class Meta:
        name = "Ec2TagsQList"

    count: Optional[int] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    list_value: List[Ec2Tags] = field(
        default_factory=list,
        metadata={
            "name": "list",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    set: List[Ec2Tags] = field(
        default_factory=list,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    add: List[Ec2Tags] = field(
        default_factory=list,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    remove: List[Ec2Tags] = field(
        default_factory=list,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    update: List[Ec2Tags] = field(
        default_factory=list,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )


@dataclass
class GcpTagsQlist:
    class Meta:
        name = "GcpTagsQList"

    count: Optional[int] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    list_value: List[GcpTags] = field(
        default_factory=list,
        metadata={
            "name": "list",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    set: List[GcpTags] = field(
        default_factory=list,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    add: List[GcpTags] = field(
        default_factory=list,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    remove: List[GcpTags] = field(
        default_factory=list,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    update: List[GcpTags] = field(
        default_factory=list,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )


@dataclass
class IbmTagsQlist:
    class Meta:
        name = "IbmTagsQList"

    count: Optional[int] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    list_value: List[IbmTags] = field(
        default_factory=list,
        metadata={
            "name": "list",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    set: List[IbmTags] = field(
        default_factory=list,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    add: List[IbmTags] = field(
        default_factory=list,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    remove: List[IbmTags] = field(
        default_factory=list,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    update: List[IbmTags] = field(
        default_factory=list,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )


@dataclass
class OciTagsQlist:
    class Meta:
        name = "OciTagsQList"

    count: Optional[int] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    list_value: List[OciTag] = field(
        default_factory=list,
        metadata={
            "name": "list",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    set: List[OciTag] = field(
        default_factory=list,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    add: List[OciTag] = field(
        default_factory=list,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    remove: List[OciTag] = field(
        default_factory=list,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    update: List[OciTag] = field(
        default_factory=list,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )


@dataclass
class OciVnicQlist:
    class Meta:
        name = "OciVnicQList"

    count: Optional[int] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    list_value: List[OciVnic] = field(
        default_factory=list,
        metadata={
            "name": "list",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    set: List[OciVnic] = field(
        default_factory=list,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    add: List[OciVnic] = field(
        default_factory=list,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    remove: List[OciVnic] = field(
        default_factory=list,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    update: List[OciVnic] = field(
        default_factory=list,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )


@dataclass
class AssetSourceQlist:
    class Meta:
        name = "AssetSourceQList"

    count: Optional[int] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    list_value: List[AssetSource] = field(
        default_factory=list,
        metadata={
            "name": "list",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    set: List[AssetSource] = field(
        default_factory=list,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    add: List[AssetSource] = field(
        default_factory=list,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    remove: List[AssetSource] = field(
        default_factory=list,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    update: List[AssetSource] = field(
        default_factory=list,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )


@dataclass
class AzureVmTags:
    tags: Optional[AzureTagsQlist] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
            "required": True,
        },
    )


@dataclass
class Ec2InstanceTags:
    tags: Optional[Ec2TagsQlist] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
            "required": True,
        },
    )


@dataclass
class GcpInstanceTags:
    tags: Optional[GcpTagsQlist] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
            "required": True,
        },
    )


@dataclass
class IbmInstanceTags:
    tags: Optional[IbmTagsQlist] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
            "required": True,
        },
    )


@dataclass
class OciInstanceTags:
    tags: Optional[OciTagsQlist] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
            "required": True,
        },
    )


@dataclass
class OciInstanceVnic:
    vnic: Optional[OciVnicQlist] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
            "required": True,
        },
    )


@dataclass
class AzureAssetSourceSimple(AssetSource):
    azure_vm_tags: Optional[AzureVmTags] = field(
        default=None,
        metadata={
            "name": "azureVmTags",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    name: Optional[str] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    location: Optional[str] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    vm_size: Optional[str] = field(
        default=None,
        metadata={
            "name": "vmSize",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    vm_id: Optional[str] = field(
        default=None,
        metadata={
            "name": "vmId",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    offer: Optional[str] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    state: Optional[AssetSourceStateCode] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    publisher: Optional[str] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    version: Optional[str] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    os_type: Optional[str] = field(
        default=None,
        metadata={
            "name": "osType",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    subnet: Optional[str] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    ipv6: Optional[str] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    subscription_id: Optional[str] = field(
        default=None,
        metadata={
            "name": "subscriptionId",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    resource_group_name: Optional[str] = field(
        default=None,
        metadata={
            "name": "resourceGroupName",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    mac_address: Optional[str] = field(
        default=None,
        metadata={
            "name": "macAddress",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    public_ip_address: Optional[str] = field(
        default=None,
        metadata={
            "name": "publicIpAddress",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    private_ip_address: Optional[str] = field(
        default=None,
        metadata={
            "name": "privateIpAddress",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    virtual_network: Optional[str] = field(
        default=None,
        metadata={
            "name": "virtualNetwork",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )


@dataclass
class Ec2AssetSourceSimple(AssetSource):
    ec2_instance_tags: Optional[Ec2InstanceTags] = field(
        default=None,
        metadata={
            "name": "ec2InstanceTags",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    reservation_id: Optional[str] = field(
        default=None,
        metadata={
            "name": "reservationId",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    availability_zone: Optional[str] = field(
        default=None,
        metadata={
            "name": "availabilityZone",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    private_dns_name: Optional[str] = field(
        default=None,
        metadata={
            "name": "privateDnsName",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    public_dns_name: Optional[str] = field(
        default=None,
        metadata={
            "name": "publicDnsName",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    local_hostname: Optional[str] = field(
        default=None,
        metadata={
            "name": "localHostname",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    instance_id: Optional[str] = field(
        default=None,
        metadata={
            "name": "instanceId",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    instance_type: Optional[str] = field(
        default=None,
        metadata={
            "name": "instanceType",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    created_date: Optional[dt.datetime] = field(
        default=None,
        metadata={
            "name": "createdDate",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
            "format": "%Y-%m-%dT%H:%M:%SZ",
        },
    )
    instance_state: Optional[AssetSourceStateCode] = field(
        default=None,
        metadata={
            "name": "instanceState",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    group_id: Optional[str] = field(
        default=None,
        metadata={
            "name": "groupId",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    group_name: Optional[str] = field(
        default=None,
        metadata={
            "name": "groupName",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    spot_instance: Optional[bool] = field(
        default=None,
        metadata={
            "name": "spotInstance",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    instance_group_id: Optional[str] = field(
        default=None,
        metadata={
            "name": "instanceGroupId",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    instance_group_name: Optional[str] = field(
        default=None,
        metadata={
            "name": "instanceGroupName",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    key_name: Optional[str] = field(
        default=None,
        metadata={
            "name": "keyName",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    launch_time: Optional[dt.datetime] = field(
        default=None,
        metadata={
            "name": "launchTime",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
            "format": "%Y-%m-%dT%H:%M:%SZ",
        },
    )
    account_id: Optional[str] = field(
        default=None,
        metadata={
            "name": "accountId",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    placement_group_name: Optional[str] = field(
        default=None,
        metadata={
            "name": "placementGroupName",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    product_code: Optional[str] = field(
        default=None,
        metadata={
            "name": "productCode",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    source_destination_check: Optional[bool] = field(
        default=None,
        metadata={
            "name": "sourceDestinationCheck",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    spot_instance_request_id: Optional[str] = field(
        default=None,
        metadata={
            "name": "spotInstanceRequestId",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    subnet_id: Optional[str] = field(
        default=None,
        metadata={
            "name": "subnetId",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    vpc_id: Optional[str] = field(
        default=None,
        metadata={
            "name": "vpcId",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    region: Optional[str] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    availibility_zone: Optional[str] = field(
        default=None,
        metadata={
            "name": "availibilityZone",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    zone: Optional[str] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    endpoint_id: Optional[int] = field(
        default=None,
        metadata={
            "name": "endpointId",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    image_id: Optional[str] = field(
        default=None,
        metadata={
            "name": "imageId",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    public_ip_address: Optional[str] = field(
        default=None,
        metadata={
            "name": "publicIpAddress",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    private_ip_address: Optional[str] = field(
        default=None,
        metadata={
            "name": "privateIpAddress",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    mac_address: Optional[str] = field(
        default=None,
        metadata={
            "name": "macAddress",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    monitoring_enabled: Optional[bool] = field(
        default=None,
        metadata={
            "name": "monitoringEnabled",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )


@dataclass
class GcpAssetSourceSimple(AssetSource):
    gcp_instance_tags: Optional[GcpInstanceTags] = field(
        default=None,
        metadata={
            "name": "gcpInstanceTags",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    instance_id: Optional[str] = field(
        default=None,
        metadata={
            "name": "instanceId",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    hostname: Optional[str] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    machine_type: Optional[str] = field(
        default=None,
        metadata={
            "name": "machineType",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    image_id: Optional[str] = field(
        default=None,
        metadata={
            "name": "imageId",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    zone: Optional[str] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    project_id_no: Optional[str] = field(
        default=None,
        metadata={
            "name": "projectIdNo",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    state: Optional[AssetSourceStateCode] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    project_id: Optional[str] = field(
        default=None,
        metadata={
            "name": "projectId",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    network: Optional[str] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    mac_address: Optional[str] = field(
        default=None,
        metadata={
            "name": "macAddress",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    public_ip_address: Optional[str] = field(
        default=None,
        metadata={
            "name": "publicIpAddress",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    private_ip_address: Optional[str] = field(
        default=None,
        metadata={
            "name": "privateIpAddress",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )


@dataclass
class IbmassetSourceSimple(AssetSource):
    class Meta:
        name = "IBMAssetSourceSimple"

    ibm_instance_tags: Optional[IbmInstanceTags] = field(
        default=None,
        metadata={
            "name": "ibmInstanceTags",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    ibm_id: Optional[int] = field(
        default=None,
        metadata={
            "name": "ibmId",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    location: Optional[str] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    datacenter_id: Optional[int] = field(
        default=None,
        metadata={
            "name": "datacenterId",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    device_name: Optional[str] = field(
        default=None,
        metadata={
            "name": "deviceName",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    state: Optional[AssetSourceStateCode] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    public_vlan: Optional[str] = field(
        default=None,
        metadata={
            "name": "publicVlan",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    domain: Optional[str] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    private_vlan: Optional[str] = field(
        default=None,
        metadata={
            "name": "privateVlan",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    public_ip: Optional[str] = field(
        default=None,
        metadata={
            "name": "publicIp",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    private_ip: Optional[str] = field(
        default=None,
        metadata={
            "name": "privateIp",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )


@dataclass
class OciAssetSourceSimple(AssetSource):
    oci_id: Optional[str] = field(
        default=None,
        metadata={
            "name": "ociId",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    display_name: Optional[str] = field(
        default=None,
        metadata={
            "name": "displayName",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    compartment_id: Optional[str] = field(
        default=None,
        metadata={
            "name": "compartmentId",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    compartment_name: Optional[str] = field(
        default=None,
        metadata={
            "name": "compartmentName",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    tenant_id: Optional[str] = field(
        default=None,
        metadata={
            "name": "tenantId",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    tenant_name: Optional[str] = field(
        default=None,
        metadata={
            "name": "tenantName",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    shape: Optional[str] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    state: Optional[AssetSourceStateCode] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    region: Optional[str] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    availability_domain: Optional[str] = field(
        default=None,
        metadata={
            "name": "availabilityDomain",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    image: Optional[str] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    fault_domain: Optional[str] = field(
        default=None,
        metadata={
            "name": "faultDomain",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    host_name: Optional[str] = field(
        default=None,
        metadata={
            "name": "hostName",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    canonical_region: Optional[str] = field(
        default=None,
        metadata={
            "name": "canonicalRegion",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    oci_instance_tags: Optional[OciInstanceTags] = field(
        default=None,
        metadata={
            "name": "ociInstanceTags",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    oci_vnic: Optional[OciInstanceVnic] = field(
        default=None,
        metadata={
            "name": "ociVnic",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )


@dataclass
class Ec2AssetSource(Ec2AssetSourceSimple):
    pass



from dataclasses import field
from typing import Optional

from pydantic.dataclasses import dataclass

from .asset_data_connector import AssetDataConnector
from .azure_service import AzureAuthRecordSimple

__NAMESPACE__ = "http://am.oxm.api.portal.qualys.com/v3"


@dataclass
class AzureAssetDataConnector(AssetDataConnector):
    auth_record: Optional[AzureAuthRecordSimple] = field(
        default=None,
        metadata={
            "name": "authRecord",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
            "required": True,
        },
    )
    subscription_name: Optional[str] = field(
        default=None,
        metadata={
            "name": "subscriptionName",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )



from dataclasses import dataclass, field
from enum import Enum
from typing import List, Optional


@dataclass
class Ip:
    class Meta:
        name = "IP"

    network_id: Optional[str] = field(
        default=None,
        metadata={
            "type": "Attribute",
        },
    )
    value: str = field(
        default="",
        metadata={
            "required": True,
        },
    )


@dataclass
class Request:
    class Meta:
        name = "REQUEST"

    name: Optional[str] = field(
        default=None,
        metadata={
            "type": "Attribute",
            "required": True,
        },
    )
    at: Optional[str] = field(
        default=None,
        metadata={
            "type": "Attribute",
            "required": True,
        },
    )
    username: Optional[str] = field(
        default=None,
        metadata={
            "type": "Attribute",
            "required": True,
        },
    )
    value: str = field(
        default="",
        metadata={
            "required": True,
        },
    )


class ResponseStatus(Enum):
    FAILED = "FAILED"
    SUCCESS = "SUCCESS"
    WARNING = "WARNING"


@dataclass
class Ignored:
    class Meta:
        name = "IGNORED"

    ticket_number: Optional[str] = field(
        default=None,
        metadata={
            "name": "TICKET_NUMBER",
            "type": "Element",
            "required": True,
        },
    )
    qid: Optional[str] = field(
        default=None,
        metadata={
            "name": "QID",
            "type": "Element",
            "required": True,
        },
    )
    ip: Optional[Ip] = field(
        default=None,
        metadata={
            "name": "IP",
            "type": "Element",
            "required": True,
        },
    )
    dns: Optional[str] = field(
        default=None,
        metadata={
            "name": "DNS",
            "type": "Element",
        },
    )
    netbios: Optional[str] = field(
        default=None,
        metadata={
            "name": "NETBIOS",
            "type": "Element",
        },
    )


@dataclass
class Restored:
    class Meta:
        name = "RESTORED"

    ticket_number: Optional[str] = field(
        default=None,
        metadata={
            "name": "TICKET_NUMBER",
            "type": "Element",
            "required": True,
        },
    )
    qid: Optional[str] = field(
        default=None,
        metadata={
            "name": "QID",
            "type": "Element",
            "required": True,
        },
    )
    ip: Optional[Ip] = field(
        default=None,
        metadata={
            "name": "IP",
            "type": "Element",
            "required": True,
        },
    )
    dns: Optional[str] = field(
        default=None,
        metadata={
            "name": "DNS",
            "type": "Element",
        },
    )
    netbios: Optional[str] = field(
        default=None,
        metadata={
            "name": "NETBIOS",
            "type": "Element",
        },
    )


@dataclass
class IgnoredList:
    class Meta:
        name = "IGNORED_LIST"

    ignored: List[Ignored] = field(
        default_factory=list,
        metadata={
            "name": "IGNORED",
            "type": "Element",
            "min_occurs": 1,
        },
    )


@dataclass
class RestoredList:
    class Meta:
        name = "RESTORED_LIST"

    restored: List[Restored] = field(
        default_factory=list,
        metadata={
            "name": "RESTORED",
            "type": "Element",
            "min_occurs": 1,
        },
    )


@dataclass
class Response:
    class Meta:
        name = "RESPONSE"

    status: Optional[ResponseStatus] = field(
        default=None,
        metadata={
            "type": "Attribute",
            "required": True,
        },
    )
    number: Optional[str] = field(
        default=None,
        metadata={
            "type": "Attribute",
        },
    )
    message: Optional[str] = field(
        default=None,
        metadata={
            "name": "MESSAGE",
            "type": "Element",
            "required": True,
        },
    )
    ignored_list: Optional[IgnoredList] = field(
        default=None,
        metadata={
            "name": "IGNORED_LIST",
            "type": "Element",
        },
    )
    restored_list: Optional[RestoredList] = field(
        default=None,
        metadata={
            "name": "RESTORED_LIST",
            "type": "Element",
        },
    )


@dataclass
class IgnoreVulnOutput:
    class Meta:
        name = "IGNORE_VULN_OUTPUT"

    request: Optional[Request] = field(
        default=None,
        metadata={
            "name": "REQUEST",
            "type": "Element",
        },
    )
    response: Optional[Response] = field(
        default=None,
        metadata={
            "name": "RESPONSE",
            "type": "Element",
            "required": True,
        },
    )



# mypy: ignore-errors
# type: ignore

import inspect
import os
import sys
import unittest

currentdir = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))
parentdir = os.path.dirname(currentdir)
sys.path.insert(0, parentdir)

from qualyspy import asset_mgmt_tagging  # noqa: E402

import qualyspy.models.asset_mgmt_tagging.tag as tag_models  # noqa: E402
from qualyspy.models.asset_mgmt_tagging import (  # noqa: E402
    azure_asset_data_connector as aadc_models,
)
from qualyspy.models.asset_mgmt_tagging import (  # noqa: E402
    asset_data_connector as adc_models,
)
from qualyspy.models.asset_mgmt_tagging import azure_service as as_models  # noqa: E402


class TestTags(unittest.TestCase):
    def test_create_update_delete_tag(self):
        api = asset_mgmt_tagging.TagsAPI()
        tag = tag_models.Tag(
            name="Test Tag",
            description="Test Description",
            children=tag_models.TagSimpleQlist(
                set=tag_models.TagSimpleList(
                    tag_simple=[
                        tag_models.TagSimple(name="Child 1"),
                        tag_models.TagSimple(name="Child 2"),
                    ]
                ),
            ),
        )
        new_tag = api.create_tag(tag=tag)
        tag = tag_models.Tag(name="Updated Test Tag")
        updated_tag = api.update_tag(id=new_tag.id, tag=tag)
        self.assertEqual(updated_tag.id, new_tag.id)
        deleted_tag = api.delete_tag(id=new_tag.id)
        self.assertEqual(deleted_tag.id, new_tag.id)

    def test_search_tags(self):
        api = asset_mgmt_tagging.TagsAPI()
        tag = tag_models.Tag(name="Test Tag", description="Test Description")
        new_tag = api.create_tag(tag=tag)
        filters = [
            asset_mgmt_tagging.Filter(field="name", operator="EQUALS", value="Test Tag")
        ]
        tags = api.search_tags(filters=filters)
        self.assertEqual(tags[0].id, new_tag.id)
        deleted_tag = api.delete_tag(id=new_tag.id)
        self.assertEqual(deleted_tag.id, new_tag.id)

    def test_count_tags(self):
        api = asset_mgmt_tagging.TagsAPI()
        tag = tag_models.Tag(name="Test Tag", description="Test Description")
        new_tag = api.create_tag(tag=tag)
        filters = [
            asset_mgmt_tagging.Filter(field="name", operator="EQUALS", value="Test Tag")
        ]
        count = api.count_tags(filters=filters)
        self.assertEqual(count, 1)
        deleted_tag = api.delete_tag(id=new_tag.id)
        self.assertEqual(deleted_tag.id, new_tag.id)


class TestAzureConnectors(unittest.TestCase):
    def test_create_delete_azure_connector(self):
        tag_api = asset_mgmt_tagging.TagsAPI()
        azure_tag = tag_api.search_tags(
            filters=[asset_mgmt_tagging.Filter("name", "EQUALS", "Microsoft Azure")]
        )
        azure_tag = tag_models.TagSimple(id=azure_tag[0].id)
        tag_qlist = tag_models.TagSimpleQlist(
            set=tag_models.TagSimpleList(tag_simple=[azure_tag])
        )

        activation_qlist = adc_models.ActivationModuleQlist(
            set=adc_models.ActivationModuleObj(
                [adc_models.ActivationModule.VM, adc_models.ActivationModule.CERTVIEW]
            )
        )

        auth_record_fields = {
            "application_id": "4d989e82-1a55-4811-8c30-14d05663ddf9",
            "directory_id": "4d989e82-1a55-4811-8c30-14d05663ddf9",
            "subscription_id": "4d989e82-1a55-4811-8c30-14d05663ddf5",
            "authentication_key": "NotARealKey",
        }
        auth_record = as_models.AzureAuthRecordSimple(**auth_record_fields)

        connector_app_infos = adc_models.ConnectorAppInfoList(
            set=adc_models.ConnectorAppInfoQlistList(
                connector_app_info_q_list=[
                    adc_models.ConnectorAppInfoQlist(
                        set=adc_models.ConnectorAppInfoObj(
                            connector_app_info=adc_models.ConnectorAppInfo(
                                name="AI",
                                identifier=auth_record_fields["subscription_id"],
                                tag_id=azure_tag.id,
                            )
                        )
                    ),
                    adc_models.ConnectorAppInfoQlist(
                        set=adc_models.ConnectorAppInfoObj(
                            connector_app_info=adc_models.ConnectorAppInfo(
                                name="CI",
                                identifier=auth_record_fields["subscription_id"],
                                tag_id=azure_tag.id,
                            )
                        )
                    ),
                ]
            )
        )

        new_aadc = aadc_models.AzureAssetDataConnector(
            name="Test Azure Connector",
            default_tags=tag_qlist,
            activation=activation_qlist,
            auth_record=auth_record,
            disabled=True,
            run_frequency=240,
            is_gov_cloud_configured=False,
            connector_app_infos=connector_app_infos,
        )

        api = asset_mgmt_tagging.AzureConnectorsAPI()
        new_connector = api.create_azure_connector(new_aadc)
        self.assertEqual(new_connector.name, new_aadc.name)
        deleted_connector = api.delete_azure_connector(new_connector.id)
        self.assertEqual(deleted_connector.id, new_connector.id)

    def test_search_azure_connector(self):
        tag_api = asset_mgmt_tagging.TagsAPI()
        azure_tag = tag_api.search_tags(
            filters=[asset_mgmt_tagging.Filter("name", "EQUALS", "Microsoft Azure")]
        )
        azure_tag = tag_models.TagSimple(id=azure_tag[0].id)
        tag_qlist = tag_models.TagSimpleQlist(
            set=tag_models.TagSimpleList(tag_simple=[azure_tag])
        )

        activation_qlist = adc_models.ActivationModuleQlist(
            set=[adc_models.ActivationModule.VM, adc_models.ActivationModule.CERTVIEW]
        )

        auth_record_fields = {
            "application_id": "4d989e82-1a55-4811-8c30-14d05663ddf9",
            "directory_id": "4d989e82-1a55-4811-8c30-14d05663ddf9",
            "subscription_id": "4d989e82-1a55-4811-8c30-14d05663ddf8",
            "authentication_key": "NotARealKey",
        }
        auth_record = as_models.AzureAuthRecordSimple(**auth_record_fields)

        connector_app_infos = adc_models.ConnectorAppInfoList(
            set=adc_models.ConnectorAppInfoQlistList(
                connector_app_info_q_list=[
                    adc_models.ConnectorAppInfoQlist(
                        set=adc_models.ConnectorAppInfoObj(
                            connector_app_info=adc_models.ConnectorAppInfo(
                                name="AI",
                                identifier=auth_record_fields["subscription_id"],
                                tag_id=azure_tag.id,
                            )
                        )
                    ),
                    adc_models.ConnectorAppInfoQlist(
                        set=adc_models.ConnectorAppInfoObj(
                            connector_app_info=adc_models.ConnectorAppInfo(
                                name="CI",
                                identifier=auth_record_fields["subscription_id"],
                                tag_id=azure_tag.id,
                            )
                        )
                    ),
                ]
            )
        )

        new_aadc = aadc_models.AzureAssetDataConnector(
            name="Test Azure Connector",
            default_tags=tag_qlist,
            activation=activation_qlist,
            auth_record=auth_record,
            disabled=True,
            run_frequency=240,
            is_gov_cloud_configured=False,
            connector_app_infos=connector_app_infos,
        )

        api = asset_mgmt_tagging.AzureConnectorsAPI()
        new_connector = api.create_azure_connector(new_aadc)
        self.assertEqual(new_connector.name, new_aadc.name)

        filters = [asset_mgmt_tagging.Filter("name", "EQUALS", "Test Azure Connector")]
        searched_connectors = api.search_azure_connectors(filters=filters)
        self.assertEqual(searched_connectors[0].id, new_connector.id)

        deleted_connector = api.delete_azure_connector(new_connector.id)
        self.assertEqual(deleted_connector.id, new_connector.id)



from . import URLS
from .base import QualysAPIBase
from .exceptions import QualysAPIError
from typing import Any

from .models.asset_mgmt_tagging import tag as tag_models
from .models.asset_mgmt_tagging import (
    azure_asset_data_connector as azure_connector_models,
)

from xsdata.formats.dataclass.serializers import JsonSerializer
from xsdata.formats.dataclass.parsers import JsonParser

import json


def _filter_none_emptylist(x: tuple[Any]) -> dict[str, Any]:
    return {k: v for k, v in x if (v is not None) and (not v == [])}


def _check_response_code(response: dict[str, Any]) -> None:
    if response["responseCode"] != "SUCCESS":
        raise QualysAPIError(response)


class PaginationSettings:
    """Pagination settings for tag search.

    Args:
        start_from_offset (int, optional): The first item to return by indedx.  The default is 1.
        start_from_id (int, optional): The first item to return by ID.  No default value.
        limit_results (int, optional): The maximum number of items to return.  The default is 100.
    """

    def __init__(
        self,
        start_from_offset: int | None = None,
        start_from_id: int | None = None,
        limit_results: int | None = None,
    ):
        self.start_from_offset = start_from_offset
        self.start_from_id = start_from_id
        self.limit_results = limit_results

    def _preferences(self) -> dict[str, dict[str, int | None]]:
        return {
            "preferences": {
                "startFromOffset": self.start_from_offset,
                "startFromId": self.start_from_id,
                "limitResults": self.limit_results,
            }
        }


class Filter:
    """A filter to apply to a tag search.

    Args:
        field (str): The field to filter on.
        operator (str): The operator to use for the filter.
        value (str): The value to filter on.
    """

    def __init__(self, field: str, operator: str, value: str):
        self.field = field
        self.operator = operator
        self.value = value

    def _as_dict(self) -> dict[str, str]:
        return {
            "field": self.field,
            "operator": self.operator,
            "value": self.value,
        }


class TagsAPI(QualysAPIBase):
    """This class contains methods for interacting with Qualys tags."""

    def _manage_tag(self, tag: tag_models.Tag, url: str) -> list[tag_models.Tag]:
        """Helper function for tag operations.

        Args:
            tag (tag_models.Tag): The tag to create or update.
            url (str): The URL to use for the request.

        Returns:
            list[tag_models.Tag]: The created or updated tag.
        """
        tag_json = JsonSerializer(dict_factory=_filter_none_emptylist).render(tag)
        tag_json = json.loads(tag_json)
        data = {"ServiceRequest": {"data": {"Tag": tag_json}}}
        response = self.post(url, data=json.dumps(data))
        ret = json.loads(response.text)
        _check_response_code(ret["ServiceResponse"])
        tags: list[tag_models.Tag] = []
        for i in range(len(ret["ServiceResponse"]["data"])):
            if "Tag" in ret["ServiceResponse"]["data"][i]:
                tags.append(tag_models.Tag(**ret["ServiceResponse"]["data"][i]["Tag"]))
        return tags

    def create_tag(self, tag: tag_models.Tag) -> tag_models.Tag:
        """Create a new tag and possibly child tags.

        Args:
            tag (tag_models.Tag): The tag to create.

        Returns:
            tag_models.Tag: The created tag.
        """
        return self._manage_tag(tag, URLS.create_tag)[0]

    def update_tag(self, id: int, tag: tag_models.Tag) -> tag_models.Tag:
        """Update fields for a tag.

        Args:
            id (int): The ID of the tag to update.
            tag (tag_models.Tag): The tag fields to update.

        Returns:
            tag_models.Tag: The updated tag.
        """
        return self._manage_tag(tag, URLS.update_tag + f"/{id}")[0]

    def search_tags(
        self,
        filters: list[Filter] | None = None,
        pagination_settings: PaginationSettings | None = None,
    ) -> list[tag_models.Tag]:
        """Returns a list of tags matching the given criteria.

        Args:
            filters (list[Filter]): The filters to apply to the search.

        Returns:
            list[tag_models.Tag]: The tags matching the given criteria.
        """
        if filters is not None:
            data = {
                "ServiceRequest": {
                    "filters": {"Criteria": [f._as_dict() for f in filters]}
                }
            }
            response = self.post(URLS.search_tags, data=json.dumps(data))
        else:
            response = self.post(URLS.search_tags)
        ret = json.loads(response.text)
        _check_response_code(ret["ServiceResponse"])
        tags: list[tag_models.Tag] = []
        for i in range(len(ret["ServiceResponse"]["data"])):
            if "Tag" in ret["ServiceResponse"]["data"][i]:
                tags.append(tag_models.Tag(**ret["ServiceResponse"]["data"][i]["Tag"]))
        return tags

    def count_tags(self, filters: list[Filter]) -> int:
        """Returns the number of tags matching the given criteria.

        Args:
            filters (list[Filter]): The filters to apply to the search.

        Returns:
            int: The number of tags matching the given criteria.
        """
        data = {
            "ServiceRequest": {"filters": {"Criteria": [f._as_dict() for f in filters]}}
        }
        response = self.post(URLS.count_tags, data=json.dumps(data))
        ret = json.loads(response.text)
        _check_response_code(ret["ServiceResponse"])
        return int(ret["ServiceResponse"]["count"])

    def delete_tag(self, id: int) -> tag_models.TagSimple:
        """Delete a tag.

        Args:
            id (int): The ID of the tag to delete.

        Returns:
            tag_models.TagSimple: The deleted tag.
        """
        response = self.post(URLS.delete_tag + f"/{id}")
        ret = json.loads(response.text)
        _check_response_code(ret["ServiceResponse"])
        return tag_models.TagSimple(**ret["ServiceResponse"]["data"][0]["Tag"])


class AzureConnectorsAPI(QualysAPIBase):
    """This class contains methods for interacting with Qualys Azure Connectors."""

    def create_azure_connector(
        self, connector: azure_connector_models.AzureAssetDataConnector
    ) -> azure_connector_models.AzureAssetDataConnector:
        """Create a new Azure connector.

        Args:
            connector (azure_connector_models.AzureAssetDataConnector): The connector to create.

        Returns:
            azure_connector_models.AzureAssetDataConnector: The created connector.
        """
        connector_json = json.loads(
            JsonSerializer(dict_factory=_filter_none_emptylist).render(connector)
        )
        data = {"ServiceRequest": {"data": {"AzureAssetDataConnector": connector_json}}}
        response = self.post(URLS.create_azure_connector, data=json.dumps(data))
        ret = json.loads(response.text)
        _check_response_code(ret["ServiceResponse"])
        if "AzureAssetDataConnector" in ret["ServiceResponse"]["data"][0]:
            new_connector = azure_connector_models.AzureAssetDataConnector(
                **ret["ServiceResponse"]["data"][0]["AzureAssetDataConnector"]
            )
        return new_connector

    def delete_azure_connector(
        self, id: int
    ) -> azure_connector_models.AzureAssetDataConnector:
        response = self.post(URLS.delete_azure_connector + f"/{id}")
        ret = json.loads(response.text)
        _check_response_code(ret["ServiceResponse"])
        return azure_connector_models.AzureAssetDataConnector(
            **ret["ServiceResponse"]["data"][0]["AzureAssetDataConnector"]
        )

    def search_azure_connectors(
        self, filters: list[Filter] | None = None
    ) -> list[azure_connector_models.AzureAssetDataConnector]:
        if filters is not None:
            data = {
                "ServiceRequest": {
                    "filters": {"Criteria": [f._as_dict() for f in filters]}
                }
            }
            response = self.post(URLS.search_azure_connector, data=json.dumps(data))
        else:
            response = self.post(URLS.search_azure_connector)
        resp = json.loads(response.text)
        _check_response_code(resp["ServiceResponse"])

        connectors: list[azure_connector_models.AzureAssetDataConnector] = []
        if "data" in resp["ServiceResponse"]:
            for i in range(len(resp["ServiceResponse"]["data"])):
                if "AzureAssetDataConnector" in resp["ServiceResponse"]["data"][i]:
                    connector_json = json.dumps(
                        resp["ServiceResponse"]["data"][i]["AzureAssetDataConnector"]
                    )
                    connectors.append(
                        JsonParser().from_string(
                            connector_json,
                            azure_connector_models.AzureAssetDataConnector,
                        )
                    )

        return connectors



from dataclasses import field
from enum import Enum
from typing import List, Optional

from pydantic.dataclasses import dataclass
from xsdata.models.datatype import XmlDate, XmlDateTime

from .tag import Tag, TagSimpleQlist

__NAMESPACE__ = "http://am.oxm.api.portal.qualys.com/v3"


class ActivationModule(Enum):
    VM = "VM"
    PC = "PC"
    SCA = "SCA"
    CLOUDVIEW = "CLOUDVIEW"
    CERTVIEW = "CERTVIEW"


@dataclass
class ActivationModuleObj:
    activation_module: list[ActivationModule] = field(
        default_factory=list,
        metadata={
            "name": "ActivationModule",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )


@dataclass
class AssetDataConnectorErrors:
    error_message: Optional[str] = field(
        default=None,
        metadata={
            "name": "errorMessage",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    created: Optional[XmlDate] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )


class AssetDataConnectorState(Enum):
    PENDING = "PENDING"
    RUNNING = "RUNNING"
    SUCCESS = "SUCCESS"
    ERROR = "ERROR"
    QUEUED = "QUEUED"
    PROCESSING = "PROCESSING"
    FINISHED_SUCCESS = "FINISHED_SUCCESS"
    FINISHED_ERRORS = "FINISHED_ERRORS"
    DISABLED = "DISABLED"
    INCOMPLETE = "INCOMPLETE"


class AssetDataConnectorType(Enum):
    AWS = "AWS"
    AZURE = "AZURE"
    OCI = "OCI"


@dataclass
class ConnectorApp:
    id: Optional[int] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    name: Optional[str] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    description: Optional[str] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )


class ConnectorScanRecurrence(Enum):
    DAILY = "DAILY"
    WEEKLY = "WEEKLY"


@dataclass
class ConnectorScanSetting:
    is_custom_scan_config_enabled: Optional[bool] = field(
        default=None,
        metadata={
            "name": "isCustomScanConfigEnabled",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )


class Day(Enum):
    SUN = "SUN"
    MON = "MON"
    TUE = "TUE"
    WED = "WED"
    THU = "THU"
    FRI = "FRI"
    SAT = "SAT"


@dataclass
class TagMetadata:
    id: Optional[int] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    name: Optional[str] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    icon: Optional[str] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    color: Optional[str] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    foreground_color: Optional[str] = field(
        default=None,
        metadata={
            "name": "foregroundColor",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    background_color: Optional[str] = field(
        default=None,
        metadata={
            "name": "backgroundColor",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )


@dataclass
class ActivationModuleQlist:
    class Meta:
        name = "ActivationModuleQList"

    count: Optional[int] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    list_value: List[ActivationModule] = field(
        default_factory=list,
        metadata={
            "name": "list",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    set: Optional[ActivationModuleObj] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    add: Optional[ActivationModuleObj] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    remove: Optional[ActivationModuleObj] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    update: Optional[ActivationModuleObj] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    activation_module: List[ActivationModule] = field(
        default_factory=list,
        metadata={
            "name": "ActivationModule",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )


@dataclass
class ConnectorAppInfo:
    name: Optional[str] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    identifier: Optional[str] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    tag_id: Optional[int] = field(
        default=None,
        metadata={
            "name": "tagId",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    tag_metadata: Optional[TagMetadata] = field(
        default=None,
        metadata={
            "name": "tagMetadata",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )


@dataclass
class ConnectorIdentifierTagInfo:
    id: Optional[int] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    tag: Optional[Tag] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )


@dataclass
class DaysOfWeekQlist:
    class Meta:
        name = "DaysOfWeekQList"

    count: Optional[int] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    list_value: List[Day] = field(
        default_factory=list,
        metadata={
            "name": "list",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    set: List[Day] = field(
        default_factory=list,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    add: List[Day] = field(
        default_factory=list,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    remove: List[Day] = field(
        default_factory=list,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    update: List[Day] = field(
        default_factory=list,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )


@dataclass
class ConnectorAppInfoObj:
    connector_app_info: Optional[ConnectorAppInfo] = field(
        default=None,
        metadata={
            "name": "ConnectorAppInfo",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )


@dataclass
class ConnectorAppInfoQlist:
    class Meta:
        name = "ConnectorAppInfoQList"

    count: Optional[int] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    list_value: List[ConnectorAppInfoObj] = field(
        default_factory=list,
        metadata={
            "name": "list",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    set: Optional[ConnectorAppInfoObj] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    add: List[ConnectorAppInfoObj] = field(
        default_factory=list,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    remove: List[ConnectorAppInfoObj] = field(
        default_factory=list,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    update: List[ConnectorAppInfoObj] = field(
        default_factory=list,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )


@dataclass
class ConnectorAppInfoQlistList:
    connector_app_info_q_list: list[ConnectorAppInfoQlist] = field(
        default_factory=list,
        metadata={
            "name": "ConnectorAppInfoQList",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )


@dataclass
class ConnectorAppInfoQlistObj:
    connector_app_info_qlist: Optional[ConnectorAppInfoQlist] = field(
        default=None,
        metadata={
            "name": "ConnectorAppInfoQList",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )


@dataclass
class ConnectorIdentifierTagInfoQlist:
    class Meta:
        name = "ConnectorIdentifierTagInfoQList"

    count: Optional[int] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    list_value: List[ConnectorIdentifierTagInfo] = field(
        default_factory=list,
        metadata={
            "name": "list",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    set: List[ConnectorIdentifierTagInfo] = field(
        default_factory=list,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    add: List[ConnectorIdentifierTagInfo] = field(
        default_factory=list,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    remove: List[ConnectorIdentifierTagInfo] = field(
        default_factory=list,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    update: List[ConnectorIdentifierTagInfo] = field(
        default_factory=list,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )


@dataclass
class ConnectorScanConfiguration:
    scan_prefix: Optional[str] = field(
        default=None,
        metadata={
            "name": "scanPrefix",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    option_profile_id: Optional[int] = field(
        default=None,
        metadata={
            "name": "optionProfileId",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    recurrence: Optional[ConnectorScanRecurrence] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    start_date: Optional[str] = field(
        default=None,
        metadata={
            "name": "startDate",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    start_time: Optional[str] = field(
        default=None,
        metadata={
            "name": "startTime",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    days_of_week: Optional[DaysOfWeekQlist] = field(
        default=None,
        metadata={
            "name": "daysOfWeek",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    timezone: Optional[str] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )


@dataclass
class ConnectorScanConfigurationObj:
    connector_scan_config: Optional[ConnectorScanConfiguration] = field(
        default=None,
        metadata={
            "name": "ConnectorScanConfiguration",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )


@dataclass
class ConnectorAppInfoList:
    count: Optional[int] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    list_value: List[ConnectorAppInfoQlistObj] = field(
        default_factory=list,
        metadata={
            "name": "list",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    set: Optional[ConnectorAppInfoQlistList] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    add: Optional[ConnectorAppInfoQlistList] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    remove: Optional[ConnectorAppInfoQlistList] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    update: Optional[ConnectorAppInfoQlistList] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )


@dataclass
class ConnectorIdentifier:
    id: Optional[int] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    identifier: Optional[str] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    connector_identifier_tag_infos: List[ConnectorIdentifierTagInfoQlist] = field(
        default_factory=list,
        metadata={
            "name": "connectorIdentifierTagInfos",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )


@dataclass
class ConnectorScanConfigQlist:
    class Meta:
        name = "ConnectorScanConfigQList"

    count: Optional[int] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    list_value: List[ConnectorScanConfigurationObj] = field(
        default_factory=list,
        metadata={
            "name": "list",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    set: List[ConnectorScanConfiguration] = field(
        default_factory=list,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    add: List[ConnectorScanConfiguration] = field(
        default_factory=list,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    remove: List[ConnectorScanConfiguration] = field(
        default_factory=list,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    update: List[ConnectorScanConfiguration] = field(
        default_factory=list,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )


@dataclass
class AssetDataConnector:
    id: Optional[int] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    name: Optional[str] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    aws_account_id: Optional[str] = field(
        default=None,
        metadata={
            "name": "awsAccountId",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    description: Optional[str] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    last_sync: Optional[XmlDateTime] = field(
        default=None,
        metadata={
            "name": "lastSync",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    last_error: Optional[str] = field(
        default=None,
        metadata={
            "name": "lastError",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    connector_state: Optional[AssetDataConnectorState] = field(
        default=None,
        metadata={
            "name": "connectorState",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    type_value: Optional[AssetDataConnectorType] = field(
        default=None,
        metadata={
            "name": "type",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    default_tags: Optional[TagSimpleQlist] = field(
        default=None,
        metadata={
            "name": "defaultTags",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    activation: Optional[ActivationModuleQlist] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    disabled: Optional[bool] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    is_gov_cloud_configured: Optional[bool] = field(
        default=None,
        metadata={
            "name": "isGovCloudConfigured",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    is_china_configured: Optional[bool] = field(
        default=None,
        metadata={
            "name": "isChinaConfigured",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    is_deleted: Optional[bool] = field(
        default=None,
        metadata={
            "name": "isDeleted",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    run_frequency: Optional[int] = field(
        default=None,
        metadata={
            "name": "runFrequency",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    next_sync: Optional[XmlDateTime] = field(
        default=None,
        metadata={
            "name": "nextSync",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    is_remediation_enabled: Optional[bool] = field(
        default=None,
        metadata={
            "name": "isRemediationEnabled",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    is_cpsenabled: Optional[bool] = field(
        default=None,
        metadata={
            "name": "isCPSEnabled",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    connector_app_infos: Optional[ConnectorAppInfoList] = field(
        default=None,
        metadata={
            "name": "connectorAppInfos",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    cloudview_uuid: Optional[str] = field(
        default=None,
        metadata={
            "name": "cloudviewUuid",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    connector_scan_setting: Optional[ConnectorScanSetting] = field(
        default=None,
        metadata={
            "name": "connectorScanSetting",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    connector_scan_config: Optional[ConnectorScanConfigQlist] = field(
        default=None,
        metadata={
            "name": "connectorScanConfig",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    is_instant_assessment_enabled: Optional[bool] = field(
        default=None,
        metadata={
            "name": "isInstantAssessmentEnabled",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    is_attached_to_org_connector: Optional[bool] = field(
        default=None,
        metadata={
            "name": "isAttachedToOrgConnector",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    is_wrapper_api: Optional[bool] = field(
        default=None,
        metadata={
            "name": "isWrapperAPI",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )


@dataclass
class ConnectorIdentifierQlist:
    class Meta:
        name = "ConnectorIdentifierQList"

    count: Optional[int] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    list_value: List[ConnectorIdentifier] = field(
        default_factory=list,
        metadata={
            "name": "list",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    set: List[ConnectorIdentifier] = field(
        default_factory=list,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    add: List[ConnectorIdentifier] = field(
        default_factory=list,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    remove: List[ConnectorIdentifier] = field(
        default_factory=list,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    update: List[ConnectorIdentifier] = field(
        default_factory=list,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )


@dataclass
class ConnectorAppCapability:
    id: Optional[int] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    connector_app: Optional[ConnectorApp] = field(
        default=None,
        metadata={
            "name": "connectorApp",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    connector_identifiers: List[ConnectorIdentifierQlist] = field(
        default_factory=list,
        metadata={
            "name": "connectorIdentifiers",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )


@dataclass
class ConnectorAppCapabilityQlist:
    class Meta:
        name = "ConnectorAppCapabilityQList"

    count: Optional[int] = field(
        default=None,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    list_value: List[ConnectorAppCapability] = field(
        default_factory=list,
        metadata={
            "name": "list",
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    set: List[ConnectorAppCapability] = field(
        default_factory=list,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    add: List[ConnectorAppCapability] = field(
        default_factory=list,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    remove: List[ConnectorAppCapability] = field(
        default_factory=list,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
    update: List[ConnectorAppCapability] = field(
        default_factory=list,
        metadata={
            "type": "Element",
            "namespace": "http://am.oxm.api.portal.qualys.com/v3",
        },
    )
